<?php
// search
// contains all search and sort parameters
// Tim Buchalka - March 2008
// Copyright (c) 2008
// Success On The Internet Pty Ltd
// PO Box 25
// Salisbury SA 5108
// Australia
// Email: tim@successontheinternet.com
//
// Version/Change History
//
// v1.0 March 2008

	require_once("error.php");
	require_once("vars.php");
	require_once("searchCondition.php");
	require_once("sortOperator.php");
	require_once("queryVars.php");
	
	class search {
	
		var $filterFields;
		var $sortFields;

		function search() {
		
			$this->filterFields = array();
			$this->sortFields = array();
		}
		
		// Accepts a searchcondition class and add's it to the array
		function addFilterLine($searchCondition) {
			$this->filterFields[] = $searchCondition;
		}
		
		// Accepts a searchcondition class and add's it to the array
		function addSortLine($sortOperator) {
			$this->sortFields[] = $sortOperator;
		}
		
		function clearFilter() {
			$this->filterFields = array();
		}
		
		function clearSort() {
			$this->sortFields = array();
		}
		
		function whereClause() {
		
			$executedOnce = 0;
			$where = "WHERE (";
			$prevJoin = "";
			foreach ( $this->filterFields as $number => $searchCondition) {
			
				if($executedOnce) {
				  $where .= ") $prevJoin (";
				}
				$fieldName = $searchCondition->getFieldName();
				$fieldType = $this->getFieldType($fieldName);
				
				$where .= $searchCondition->getFieldName() . " ";
				$oper = $searchCondition->getOperator();
				$where .= $oper . " ";
				if($oper == "between") {
					$where .= $this->preAndPosfixValue($fieldType) 
					. $searchCondition->getValue1() . $this->preAndPosfixValue($fieldType) 
					. " AND " . $this->preAndPosfixValue($fieldType) 
					. $searchCondition->getValue2() . $this->preAndPosfixValue($fieldType);

				} else if ($oper == "like") {
					$where .= $this->preAndPosfixValue($fieldType) . "%"
					. $searchCondition->getValue1() . "%" . $this->preAndPosfixValue($fieldType);
				} else {
					$where .= $this->preAndPosfixValue($fieldType) . $searchCondition->getValue1()
					. $this->preAndPosfixValue($fieldType);
				}
				
				$prevJoin = $searchCondition->getJoin();
				if(!$executedOnce) {
					$executedOnce = 1;
				}
			}

			// In case there are no fields return a basic "true" where clause
			if ($where == "WHERE (")
				return "1=1)";
				
			return $where .= ")";
		}
		
		function sortClause() {

			$orderBy = "";
			$executedOnce = 0;
			
			foreach ( $this->sortFields as $number => $sortOperator) {
				if($executedOnce) {
					$orderBy .= ", ";
				} else {
					$orderBy .= " ORDER BY ";
				}
				$orderBy .= $sortOperator->getFieldName() . " " . $sortOperator->getOperator();
				if(!$executedOnce) $executedOnce = 1;
			}

			return $orderBy;
		}		
		
		// Returns the type of the field based on the name
		// returns NULL if not found
		function getFieldType($fname) {
			global $queryFieldNames;
			
			foreach ( $queryFieldNames as $fieldName => $fieldType) {
				if ($fieldName == $fname) return $fieldType;
			}
			
			return NULL;
		}
		
		function preAndPosfixValue($fieldType) {
			if($fieldType == "string" or $fieldType == "date") return "'";
			
			return "";
		}
		
		function sqlStatement() {
			$sql = "SELECT * FROM transactions "
				. $this->whereClause() . " " . $this->sortClause();

			return $sql;
		}

	}
?>